
#==============================================================================#
# gyokuro/parts.rb
#==============================================================================#

#==============================================================================#
# Gyokuro Class
class Gyokuro

  class Parts
  end

  class Noun < Parts                   # 名詞
    General    = Class.new(self)       #  一般
    ProperNoun = Class.new(self)       #  固有名詞
    Pronoun    = Class.new(self)       #  代名詞
    Adverb     = Class.new(self)       #  副詞可能
    Number     = Class.new(self)       #  数
    Special    = Class.new(self)       #  特殊
    Suffix     = Class.new(self)       #  接尾
  end

  class Prefix < Parts                 # 接頭詞
  end

  class Verb < Parts                   # 動詞
    Independence   = Class.new(self)   #  自立
    UnIndependence = Class.new(self)   #  非自立
    Suffix         = Class.new(self)   #  接尾
  end

  class Adjective < Parts              # 形容詞
    Independence   = Class.new(self)   #  自立
    UnIndependence = Class.new(self)   #  非自立
    Suffix         = Class.new(self)   #  接尾
  end

  class Adverb < Parts                 # 副詞
    General = Class.new(self)          #  一般
  end

  class Attributive < Parts            # 連体詞
  end

  class Conjunction < Parts            # 接続詞
  end

  class Particle < Parts               # 助詞
  end

  class AuxiliaryVerb < Parts          # 助動詞
  end

  class Exclamation < Parts            # 感動詞
  end

  class Mark < Parts                   # 記号
    General          = Class.new(self) #  一般
    Period           = Class.new(self) #  ピリオド
    Comma            = Class.new(self) #  カンマ
    Space            = Class.new(self) #  空白
    Alphabet         = Class.new(self) #  アルファベット
    OpenParenthesis  = Class.new(self) #  開き括弧
    CloseParenthesis = Class.new(self) #  閉じ括弧
  end

  class Other < Parts                  # その他
  end

  class Filler < Parts                 # フィラー
  end

  class NonLanguage < Parts            # 非言語音
  end

  class Fragment < Parts               # 語断片
  end

  class Unknown < Parts                # 不明
  end

end # Gyokuro

#==============================================================================#
#==============================================================================#
